@@ -2,6 +2,8 @@
 using System;
 using System.IO;
 using System.Collections.Generic;
+using System.Threading;
+
 using Mono.Unix;
 
 namespace Tomboy
@@ -20,16 +22,6 @@ namespace Tomboy
 
 		static string start_note_uri = String.Empty;
 
-		static NoteManager ()
-		{
-			// Watch the START_NOTE_URI setting and update it so that the
-			// StartNoteUri property doesn't generate a call to
-			// Preferences.Get () each time it's accessed.
-			start_note_uri =
-			        Preferences.Get (Preferences.START_NOTE_URI) as string;
-			Preferences.SettingChanged += OnSettingChanged;
-		}
-
 		static void OnSettingChanged (object sender, NotifyEventArgs args)
 		{
 			switch (args.Key) {
@@ -39,24 +31,108 @@ namespace Tomboy
 			}
 		}
 
-public NoteManager (string directory) :
-		this (directory, Path.Combine (directory, "Backup"))
+		public NoteManager (string directory) :
+			this (directory, Path.Combine (directory, "Backup"))
 		{
 		}
 
 		public NoteManager (string directory, string backup_directory)
 		{
-			Logger.Log ("NoteManager created with note path \"{0}\".", directory);
+			Logger.Debug ("NoteManager created with note path \"{0}\".", directory);
 
 			notes_dir = directory;
 			backup_dir = backup_directory;
+		}
+
+		public bool Initialized {
+			get; private set;
+		}
+
+		// TODO: Decide if/how to enforce
+		public bool ReadOnly {
+			get; set;
+		}
+
+		/// <summary>
+		/// Use Gtk.Application.Invoke to invoke the Action delegate
+		/// once this NoteManager is initialized. If this NoteManager
+		/// is already initialized, Gtk.Application.Invoke is *not*
+		/// used (for performance reasons).  In other words, this method
+		/// should only be called from the GTK+ main thread.
+		/// </summary>
+		public void GtkInvoke (Action a)
+		{
+			if (!Initialized)
+				new Thread (() => {
+					while (!Initialized)
+						;
+					Gtk.Application.Invoke ((o, e) => a ());
+				}).Start ();
+			else
+				a ();
+		}
+
+		public void Invoke (Action a)
+		{
+			if (!Initialized)
+				new Thread (() => {
+					while (!Initialized)
+						;
+					a ();
+				}).Start ();
+			else
+				a ();
+		}
+
+		public void Initialize ()
+		{
 			notes = new List<Note> ();
 
+			string conf_dir = Services.NativeApplication.ConfigurationDirectory;
+
+			string old_notes_dir = null;
+			bool migration_needed = false;
+
 			bool first_run = FirstRun ();
+			if (first_run) {
+				old_notes_dir = Services.NativeApplication.PreOneDotZeroNoteDirectory;
+				migration_needed = DirectoryExists (old_notes_dir);
+			}
 			CreateNotesDir ();
+			if (!Directory.Exists (conf_dir))
+				Directory.CreateDirectory (conf_dir);
+
+			if (migration_needed) {
+				// Copy notes
+				foreach (string noteFile in Directory.GetFiles (old_notes_dir, "*.note"))
+					File.Copy (noteFile, Path.Combine (notes_dir, Path.GetFileName (noteFile)));
+
+				// Copy deleted notes
+				string old_backup = Path.Combine (old_notes_dir, "Backup");
+				if (DirectoryExists (old_backup)) {
+					Directory.CreateDirectory (backup_dir);
+					foreach (string noteFile in Directory.GetFiles (old_backup, "*.note"))
+						File.Copy (noteFile, Path.Combine (backup_dir, Path.GetFileName (noteFile)));
+				}
+
+				// Copy configuration data
+				// NOTE: Add-in configuration data copied by AddinManager
+				string sync_manifest_name = "manifest.xml";
+				string old_sync_manifest_path =
+					Path.Combine (old_notes_dir, sync_manifest_name);
+				if (File.Exists (old_sync_manifest_path))
+					File.Copy (old_sync_manifest_path,
+					           Path.Combine (conf_dir, sync_manifest_name));
+
+
+				// NOTE: Not copying cached data
+
+				first_run = false;
+			}
 
 			trie_controller = CreateTrieController ();
-			addin_mgr = CreateAddinManager ();
+			addin_mgr = new AddinManager (conf_dir,
+			                              migration_needed ? old_notes_dir : null);
 
 			if (first_run) {
 				// First run. Create "Start Here" notes.
@@ -65,7 +141,62 @@ public NoteManager (string directory) :
 				LoadNotes ();
 			}
 
+			if (migration_needed) {
+				// Create migration notification note
+				// Translators: The title of the data migration note
+				string base_migration_note_title = Catalog.GetString ("Your Notes Have Moved!");
+				string migration_note_title = base_migration_note_title;
+
+				// NOTE: Uncomment to generate a title suitable for
+				//       putting in the note collection
+//				int count = 1;
+//				while (Find (migration_note_title) != null) {
+//					migration_note_title = base_migration_note_title +
+//						string.Format (" ({0})", ++count);
+//				}
+
+				string migration_note_content_template = Catalog.GetString (
+// Translators: The contents (not including the title) of the data migration note. {0}, {1}, {2}, {3}, and {4} are replaced by directory paths and should not be changed
+@"In the latest version of Tomboy, your note files have moved.  You have probably never cared where your notes are stored, and if you still don't care, please go ahead and <bold>delete this note</bold>.  :-)
+
+Your old note directory is still safe and sound at <link:url>{0}</link:url> . If you go back to an older version of Tomboy, it will look for notes there.
+
+But we've copied your notes and configuration info into new directories, which will be used from now on:
+
+<list><list-item dir=""ltr"">Notes can now be found at <link:url>{1}</link:url>
+</list-item><list-item dir=""ltr"">Configuration is at <link:url>{2}</link:url>
+</list-item><list-item dir=""ltr"">You can install add-ins at <link:url>{3}</link:url>
+</list-item><list-item dir=""ltr"">Log files can be found at <link:url>{4}</link:url></list-item></list>
+
+Ciao!");
+				string migration_note_content =
+					"<note-content version=\"1.0\">" +
+					migration_note_title + "\n\n" +
+					string.Format (migration_note_content_template,
+					               old_notes_dir + Path.DirectorySeparatorChar,
+					               notes_dir + Path.DirectorySeparatorChar,
+					               conf_dir + Path.DirectorySeparatorChar,
+					               Path.Combine (conf_dir, "addins") + Path.DirectorySeparatorChar,
+					               Services.NativeApplication.LogDirectory + Path.DirectorySeparatorChar) +
+					"</note-content>";
+
+				// NOTE: Uncomment to create a Tomboy note and
+				//       show it to the user
+//				Note migration_note = Create (migration_note_title,
+//				                              migration_note_content);
+//				migration_note.QueueSave (ChangeType.ContentChanged);
+//				migration_note.Window.Show ();
+
+				// Place file announcing migration in old note directory
+				try {
+					using (var writer = File.CreateText (Path.Combine (old_notes_dir,
+					                                                   migration_note_title.ToUpper ().Replace (" ", "_"))))
+						writer.Write (migration_note_content);
+				} catch {}
+			}
+
 			Tomboy.ExitingEvent += OnExitingEvent;
+			Initialized = true;
 		}
 		
 		
@@ -75,13 +206,6 @@ public NoteManager (string directory) :
 			return new TrieController (this);
 		}
 
-		protected virtual AddinManager CreateAddinManager ()
-		{
-			string tomboy_conf_dir = Services.NativeApplication.ConfDir;
-
-			return new AddinManager (tomboy_conf_dir);
-		}
-
 		// For overriding in test methods.
 		protected virtual bool DirectoryExists (string directory)
 		{
@@ -122,6 +246,12 @@ public NoteManager (string directory) :
 			this.notes.Sort (new CompareDates ());
 		}
 
+		void OnBufferChanged (Note note)
+		{
+			if (NoteBufferChanged != null)
+				NoteBufferChanged (note);
+		}
+
 		protected virtual void CreateStartNotes ()
 		{
 			// FIXME: Delay the creation of the start notes so the panel/tray
@@ -175,7 +305,8 @@ public NoteManager (string directory) :
 				                          links_note_content);
 				links_note.QueueSave (ChangeType.ContentChanged);
 
-				start_note.Window.Show ();
+				if (!Tomboy.IsPanelApplet)
+					start_note.Window.Show ();
 			} catch (Exception e) {
 				Logger.Warn ("Error creating start notes: {0}\n{1}",
 				             e.Message, e.StackTrace);
@@ -192,10 +323,11 @@ public NoteManager (string directory) :
 					if (note != null) {
 						note.Renamed += OnNoteRename;
 						note.Saved += OnNoteSave;
+						note.BufferChanged += OnBufferChanged;
 						notes.Add (note);
 					}
 				} catch (System.Xml.XmlException e) {
-					Logger.Log ("Error parsing note XML, skipping \"{0}\": {1}",
+					Logger.Error ("Error parsing note XML, skipping \"{0}\": {1}",
 					            file_path,
 					            e.Message);
 				}
@@ -251,7 +383,7 @@ public NoteManager (string directory) :
 				}
 			}
 
-			Logger.Log ("Saving unsaved notes...");
+			Logger.Debug ("Saving unsaved notes...");
 			
 			// Use a copy of the notes to prevent bug #510442 (crash on exit
 			// when iterating the notes to save them.
@@ -287,7 +419,7 @@ public NoteManager (string directory) :
 			notes.Remove (note);
 			note.Delete ();
 
-			Logger.Log ("Deleting note '{0}'.", note.Title);
+			Logger.Debug ("Deleting note '{0}'.", note.Title);
 
 			if (NoteDeleted != null)
 				NoteDeleted (this, note);
@@ -422,6 +554,7 @@ public NoteManager (string directory) :
 			new_note.XmlContent = xml_content;
 			new_note.Renamed += OnNoteRename;
 			new_note.Saved += OnNoteSave;
+			new_note.BufferChanged += OnBufferChanged;
 
 			notes.Add (new_note);
 
@@ -512,6 +645,15 @@ public NoteManager (string directory) :
 		public static string StartNoteUri
 		{
 			get {
+				if (String.IsNullOrEmpty (start_note_uri)) {
+					// Watch the START_NOTE_URI setting and update it so that the
+					// StartNoteUri property doesn't generate a call to
+					// Preferences.Get () each time it's accessed.
+					start_note_uri =
+					        Preferences.Get (Preferences.START_NOTE_URI) as string;
+					Preferences.SettingChanged -= OnSettingChanged;
+					Preferences.SettingChanged += OnSettingChanged;
+				}
 				return start_note_uri;
 			}
 		}
@@ -551,6 +693,7 @@ public NoteManager (string directory) :
 		public event NotesChangedHandler NoteAdded;
 		public event NoteRenameHandler NoteRenamed;
 		public event NoteSavedHandler NoteSaved;
+		public event Action<Note> NoteBufferChanged;
 	}
 
 	public class TrieController
